function [imseq_seg] = SILK_1SVM(avifilename)
%
%	SILK 1-SVM (with truncated decay factor) 
%
% Notice:
%	the pxls are independant of each others
%
% Input:
%	avifilename		: filename of the input video data
%
% Output:
%	imseq_seg		: [imh,imw,    nFrames] each is a binary fg/bk image
%
%
% Remark:
%	based on 
%		1. Sparse Online Learning with kernels (SILK)
%

global imh imw dim nTrImgs trimgs nFrames imseq
global radius C_train C_tst_large C_tst_small threshold trunc_len tau epsilon
global imseq_seg  imseq_loss
global alpha_seq

% 
% -----------/// Init ///-------------
%
%	nFrames			: N,			#images
%	mov				: [1..nFrames]	#images
%	imseq			: [imh, imw, dim, nFrames]   #images
%
%	radius			: R+,	radius term for 1-svm
%
%	alpha_seq		: [imh, imw, trunc_len], each stores a value in [0,1]
%

mov						= aviread(avifilename);
mov_start				= 1;
nAll					= length(mov);
nTrImgs					= mov_start-1;					% [1..nTrImgs], #background images
nFrames					= nAll-mov_start+1;				% [1..nFrames], #images when the foreground obj. is supposed to appear
[imh,imw,dim]			= size(mov(1).cdata);
trimgs					= zeros(imh, imw, dim, nTrImgs);
imseq					= zeros(imh, imw, dim, nFrames);
for bk_i = 1:nTrImgs
	trimgs(:,:,:,bk_i)	= double(mov(bk_i).cdata);
end % frm_i
for frm_i = 1:nFrames
	imseq(:,:,:,frm_i)	= double(mov(frm_i+nTrImgs).cdata);
end % frm_i
clear mov

ext_name			= '';							% the extention of output filename is empty by default
C                   = 0.08;
C_train				= C;
C_tst_small			= C;
C_tst_large			= C;
threshold			= 0.98;
trunc_len			= 20;
tau					= 0.55;
sigma				= 12*sqrt(3);
radius					= 1.0;
epsilon					= 1e-5;
kerIdentifier			= 'rbf';
kernel_arg				= [sigma, 0];
bWant_alpha_seq			= 0;


% 
% -----------/// online update of the dual form of 1-SVM, and classification of FG/BG ///-------------
%
% Output:
%	imseq_seg		: (imh, imw, nFrames) each is a binary fg/bk image
%	alpha_seq		: (imh, imw, trunc_len)
%

if bWant_alpha_seq,
	alpha_seq				= zeros(imh, imw, trunc_len);
end
imseq_seg					= zeros(imh, imw, nFrames);
imseq_loss					= zeros(imh, imw, nFrames);
SILK_C(kerIdentifier, kernel_arg, bWant_alpha_seq);


% 
% -----------/// output ///-------------
% 

[pathstr, name, ext, versn]...
						= fileparts(avifilename);
out_avifilename...
						= sprintf('%s_SILK_%s',  name, ext_name);
mov_res					= make_avi(imseq_seg, out_avifilename);


% ----------------
% ----------------
%function [res] = kernel(x_t, x_i)


% ----------------
function [mov] = make_avi(imseq, out_avifilename)
%
% produce a avi video from image sequence
%


mov					= avifile(out_avifilename, 'compression', 'None');
if ndims(imseq)==3
	dim				= 1;
	[imh, imw, nFrms]	...
					= size(imseq);
else													% if ndims(imseq)==4
	[imh, imw, dim, nFrms]	...
					= size(imseq);
end

for i=1:nFrms
	if dim==1
		tmp			= imseq(:,:,i);
		colorim		= repmat( tmp, [1,1,3]);
	else
		colorim		= imseq(:,:,:,i);
	end
	max_val			= max(colorim(:));
	if max_val>1
		colorim		= colorim / max_val;
	end
	frm				= im2frame(colorim);
	mov				= addframe(mov, frm );
end
mov					= close(mov);


